<?php

namespace Drupal\micro_simple_sitemap\Controller;

use Drupal\Core\PageCache\ResponsePolicy\KillSwitch;
use Drupal\Core\Site\Settings;
use Drupal\micro_simple_sitemap\MicroSimpleSitemap;
use Drupal\micro_site\Entity\SiteInterface;
use Drupal\simple_sitemap\Simplesitemap;
use Drupal\Core\Cache\CacheableResponse;
use Drupal\Core\Controller\ControllerBase;
use Drupal\micro_site\SiteNegotiatorInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;
use Drupal\simple_sitemap\Controller\SimplesitemapController;
use Drupal\Core\State\StateInterface;

/**
 * Class MicroSimplesitemapController.
 *
 * @package Drupal\micro_simple_sitemap\Controller
 */
class MicroSimpleSitemapController extends SimplesitemapController {

  /**
   * The micro site negotiator.
   *
   * @var \Drupal\micro_site\SiteNegotiatorInterface
   */
  protected $negotiator;

  /**
   * The state service.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * MicroSimpleSitemapController constructor.
   *
   * @param \Drupal\micro_simple_sitemap\MicroSimpleSitemap $generator
   * @param \Drupal\Core\PageCache\ResponsePolicy\KillSwitch $cache_kill_switch
   * @param \Drupal\micro_site\SiteNegotiatorInterface $site_negotiator
   * @param \Drupal\Core\State\StateInterface $state
   */
  public function __construct(MicroSimpleSitemap $generator, KillSwitch $cache_kill_switch, SiteNegotiatorInterface $site_negotiator, StateInterface $state) {
    parent::__construct($generator, $cache_kill_switch);
    $this->negotiator = $site_negotiator;
    $this->state = $state;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('micro_simple_sitemap.generator'),
      $container->get('page_cache_kill_switch'),
      $container->get('micro_site.negotiator'),
      $container->get('state')
    );
  }

  /**
   * Returns the whole sitemap, requested sitemap chunk or sitemap index file.
   *
   * @param int $chunk_id
   *   Optional ID of the sitemap chunk. If none provided, the first chunk or
   *   the sitemap index is fetched.
   *
   * @throws NotFoundHttpException
   *
   * @return object
   *   Returns an XML response.
   */
  public function getSitemap($chunk_id = NULL) {
    // Get active site.
    $site = $this->negotiator->getActiveSite();
    if (!$site instanceof SiteInterface) {
      $response = parent::getSitemap($chunk_id);
      $meta_data = $response->getCacheableMetadata();
      $meta_data->addCacheContexts(['url.site']);
      $response->addCacheableDependency($meta_data);
      return $response;
    }

    $output = $this->state->get('micro_simple_sitemap:' . $site->id(), NULL);
    if (!$output) {
      $this->cacheKillSwitch->trigger();
      throw new NotFoundHttpException();
    }

    // Display sitemap with correct XML header.
    $response = new CacheableResponse($output, Response::HTTP_OK, [
      'content-type' => 'application/xml',
      'X-Robots-Tag' => 'noindex', // Do not index the sitemap itself.
    ]);

    // Cache output.
    $meta_data = $response->getCacheableMetadata();
    $meta_data->addCacheTags(['micro_simple_sitemap:' . $site->id() ]);
    $meta_data->addCacheContexts(['url.site']);
    $response->addCacheableDependency($meta_data);
    return $response;
  }

  public function rebuild($hash) {
    $hasl_salt = Settings::get('hash_salt');
    if (empty($hash) || $hash !== $hasl_salt) {
      throw new AccessDeniedHttpException();
    }

    $site = $this->negotiator->getActiveSite();
    if ($site instanceof SiteInterface) {
      $this->generator->generateSitemap('nobatch', $site);
      $response = new Response('OK', Response::HTTP_OK, [
        'X-Robots-Tag' => 'noindex', // Do not index the sitemap itself.
      ]);
      return $response;
    }
    throw new NotFoundHttpException();
  }
}
