<?php

namespace Drupal\micro_simple_sitemap;

use Drupal\Component\Datetime\Time;
use Drupal\micro_site\Entity\SiteInterface;
use Drupal\micro_site\SiteNegotiatorInterface;
use Drupal\simple_sitemap\Batch;
use Drupal\simple_sitemap\EntityHelper;
use Drupal\Core\Database\Connection;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Path\PathValidator;
use Drupal\Core\Entity\Query\QueryFactory;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\Datetime\DateFormatter;
use Drupal\simple_sitemap\Plugin\simple_sitemap\UrlGenerator\UrlGeneratorManager;
use Drupal\simple_sitemap\Simplesitemap;
use Drupal\simple_sitemap\SitemapGenerator;

/**
 * Class MicroSimpleSitemap.
 *
 * @package Drupal\micro_simple_sitemap
 */
class MicroSimpleSitemap extends Simplesitemap {

  /**
   * The micro site negotiator.
   *
   * @var \Drupal\micro_site\SiteNegotiatorInterface
   */
  protected $negotiator;

  /**
   * MicroSimpleSitemap constructor.
   *
   * @param \Drupal\micro_simple_sitemap\MicroSimpleSitemapGenerator $sitemapGenerator
   * @param \Drupal\simple_sitemap\EntityHelper $entityHelper
   * @param \Drupal\Core\Config\ConfigFactory $configFactory
   * @param \Drupal\Core\Database\Connection $database
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   * @param \Drupal\Core\Path\PathValidator $pathValidator
   * @param \Drupal\Core\Datetime\DateFormatter $dateFormatter
   * @param \Drupal\Component\Datetime\Time $time
   * @param \Drupal\micro_simple_sitemap\MicroBatch $batch
   * @param \Drupal\simple_sitemap\Plugin\simple_sitemap\UrlGenerator\UrlGeneratorManager $urlGeneratorManager
   * @param \Drupal\micro_site\SiteNegotiatorInterface $micro_site_negotiator
   */
  public function __construct(MicroSimpleSitemapGenerator $sitemapGenerator, EntityHelper $entityHelper, ConfigFactory $configFactory, Connection $database, EntityTypeManagerInterface $entityTypeManager, PathValidator $pathValidator, DateFormatter $dateFormatter, Time $time, MicroBatch $batch, UrlGeneratorManager $urlGeneratorManager, SiteNegotiatorInterface $micro_site_negotiator) {
    parent::__construct($sitemapGenerator, $entityHelper, $configFactory, $database, $entityTypeManager, $pathValidator, $dateFormatter, $time, $batch, $urlGeneratorManager);
    $this->negotiator = $micro_site_negotiator;
  }

  /**
   * Generates the XML sitemap and saves it to the db.
   *
   * @param string $from
   *   Can be 'form', 'backend', 'drush' or 'nobatch'.
   *   This decides how the batch process is to be run.
   *
   * @return bool|\Drupal\simple_sitemap\Simplesitemap
   */
  public function generateSitemap($from = 'form', SiteInterface $site = NULL) {
    if ($site instanceof SiteInterface) {
      $this->batch->setBatchSettings([
        'base_url' => $site->getSitePath(),
        'batch_process_limit' => $this->getSetting('batch_process_limit', NULL),
        'max_links' => 100000,
        'skip_untranslated' => $this->getSetting('skip_untranslated', FALSE),
        'remove_duplicates' => $this->getSetting('remove_duplicates', TRUE),
        'excluded_languages' => $this->getSetting('excluded_languages', []),
        'from' => $from,
        'site' => $site,
      ]);
    }
    else {
      $this->batch->setBatchSettings([
        'base_url' => $this->getSetting('base_url', ''),
        'batch_process_limit' => $this->getSetting('batch_process_limit', NULL),
        'max_links' => $this->getSetting('max_links', 2000),
        'skip_untranslated' => $this->getSetting('skip_untranslated', FALSE),
        'remove_duplicates' => $this->getSetting('remove_duplicates', TRUE),
        'excluded_languages' => $this->getSetting('excluded_languages', []),
        'from' => $from,
      ]);
    }


    $plugins = $this->urlGeneratorManager->getDefinitions();

    usort($plugins, function($a, $b) {
      return $a['weight'] - $b['weight'];
    });

    foreach ($plugins as $plugin) {
      if ($plugin['enabled']) {
        if (!empty($plugin['settings']['instantiate_for_each_data_set'])) {
          foreach ($this->urlGeneratorManager->createInstance($plugin['id'])->getDataSets() as $data_sets) {
            $this->batch->addOperation($plugin['id'], $data_sets);
          }
        }
        else {
          $this->batch->addOperation($plugin['id']);
        }
      }
    }

    $success = $this->batch->start();
    return $from === 'nobatch' ? $this : $success;
  }

}
